import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class Main {
    private static boolean checkArgs(String[] args) {

        if (args.length == 0) {
            System.out.println("Передайте путь к существующему файлу");
            return false;
        }

        Path file = Paths.get(args[0]);

        if (Files.notExists(file) || Files.isDirectory(file)) {
            System.out.println("Передайте путь к существующему файлу");
            return false;
        }

        return true;
    }

    private static Comparator<Map.Entry<String, Long>> myComparator() {
        //    вот это нужно чтобы java поняла какие типы у getValue и getKey
        //    |___________________________  ______________________________|
        //                                \/
        return Comparator.<Map.Entry<String, Long>>comparingLong(Map.Entry::getValue) // сравнивает сначала по значению количества
                .reversed() // перевернем в обратном порядке, чтобы шли по убыванию значения количества
                .thenComparing(Map.Entry::getKey); // и в конце пересортируем при равных количествах по алфавиту слова
    }

    public static void main(String[] args) throws IOException {
        if (checkArgs(args)) {
            Path file = Paths.get(args[0]);

            List<String> words = Files.lines(file)
                    .map(line -> line.split("\\s+")) // получаем массив слов в строке
                    .flatMap(Arrays::stream) // меняем массивы слов на один поток слов
                    .filter(word -> !word.isEmpty()) // убираем пустые строки из потока
                    .map(String::toLowerCase) // переводим каждое слово в нижний регистр
                    // групируем слова по их имени и считаем с помощью композиции коллекторов сколько раз слово встретилось
                    .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()))
                    // так как на этот момент уже будет Map, то получаем поток пар: слово - количество
                    .entrySet().stream()
                    .sorted(myComparator()) // сортируем пары по нашему правилу
                    .limit(10) // берем первые 10 часто встречаемых слов
                    .map(Map.Entry::getKey) // из потока пар возьмем только ключ (слово)
                    .collect(Collectors.toList()); // и преобразуем поток к списку

            // Выводим ответ к задаче

            System.out.println("Ответ на условие задачи: ");

            if (words.size() == 0) {
                System.out.println("Файл не содержит слов");
                return;
            }

            words.forEach(System.out::println);
        }
    }
}
